@@ -7,11 +7,11 @@ arg = {}
 ---#DES 'assert'
 ---@generic T
 ---@param v T
----@param message any
+---@param message? any
 ---@return T
 function assert(v, message) end
 
----@alias cgopt
+---@alias gcoptions
 ---|>'"collect"'      # ---#DESTAIL 'cgopt.collect'
 ---| '"stop"'         # ---#DESTAIL 'cgopt.stop'
 ---| '"restart"'      # ---#DESTAIL 'cgopt.restart'
@@ -28,12 +28,12 @@ function assert(v, message) end
 
 ---#if VERSION >= 5.4 then
 ---#DES 'collectgarbage'
----@param opt? cgopt
+---@param opt? gcoptions
 ---@return any
 function collectgarbage(opt, ...) end
 ---#else
 ---#DES 'collectgarbage'
----@param opt? cgopt
+---@param opt? gcoptions
 ---@param arg? integer
 ---@return any
 function collectgarbage(opt, arg) end
@@ -55,13 +55,15 @@ _G = {}
 
 ---@version 5.1
 ---#DES 'getfenv'
----@param f? function
+---@param f? async fun()
 ---@return table
+---@nodiscard
 function getfenv(f) end
 
 ---#DES 'getmetatable'
 ---@param object any
 ---@return table metatable
+---@nodiscard
 function getmetatable(object) end
 
 ---#DES 'ipairs'
@@ -83,6 +85,7 @@ function ipairs(t) end
 ---@param chunkname? string
 ---@return function
 ---@return string   error_message
+---@nodiscard
 function load(func, chunkname) end
 ---#else
 ---#DES 'load>5.2'
@@ -92,6 +95,7 @@ function load(func, chunkname) end
 ---@param env?       table
 ---@return function
 ---@return string   error_message
+---@nodiscard
 function load(chunk, chunkname, mode, env) end
 ---#end
 
@@ -100,6 +104,7 @@ function load(chunk, chunkname, mode, env) end
 ---@param filename? string
 ---@return function
 ---@return string   error_message
+---@nodiscard
 function loadfile(filename) end
 ---#else
 ---#DES 'loadfile'
@@ -108,6 +113,7 @@ function loadfile(filename) end
 ---@param env?      table
 ---@return function
 ---@return string   error_message
+---@nodiscard
 function loadfile(filename, mode, env) end
 ---#end
 
@@ -117,8 +123,15 @@ function loadfile(filename, mode, env) end
 ---@param chunkname? string
 ---@return function
 ---@return string error_message
+---@nodiscard
 function loadstring(text, chunkname) end
 
+---@version 5.1
+---@param proxy boolean|table
+---@return userdata
+---@nodiscard
+function newproxy(proxy) end
+
 ---@version 5.1
 ---#DES 'module'
 ---@param name string
@@ -130,6 +143,7 @@ function module(name, ...) end
 ---@param index? K
 ---@return K
 ---@return V
+---@nodiscard
 function next(table, index) end
 
 ---#DES 'pairs'
@@ -140,7 +154,11 @@ function next(table, index) end
 function pairs(t) end
 
 ---#DES 'pcall'
+---#if VERSION == 5.1 and not JIT then
 ---@param f     function
+---#else
+---@param f     async fun()
+---#end
 ---@param arg1? any
 ---@return boolean success
 ---@return any result
@@ -154,17 +172,20 @@ function print(...) end
 ---@param v1 any
 ---@param v2 any
 ---@return boolean
+---@nodiscard
 function rawequal(v1, v2) end
 
 ---#DES 'rawget'
 ---@param table table
 ---@param index any
 ---@return any
+---@nodiscard
 function rawget(table, index) end
 
 ---#DES 'rawlen'
 ---@param v table|string
 ---@return integer len
+---@nodiscard
 function rawlen(v) end
 
 ---#DES 'rawset'
@@ -177,11 +198,12 @@ function rawset(table, index, value) end
 ---#DES 'select'
 ---@param index integer|'"#"'
 ---@return any
+---@nodiscard
 function select(index, ...) end
 
 ---@version 5.1
 ---#DES 'setfenv'
----@param f     function|integer
+---@param f     async fun()|integer
 ---@param table table
 ---@return function
 function setfenv(f, table) end
@@ -196,11 +218,13 @@ function setmetatable(table, metatable) end
 ---@param e     string|number
 ---@param base? integer
 ---@return number?
+---@nodiscard
 function tonumber(e, base) end
 
 ---#DES 'tostring'
 ---@param v any
 ---@return string
+---@nodiscard
 function tostring(v) end
 
 ---@alias type
@@ -216,6 +240,7 @@ function tostring(v) end
 ---#DES 'type'
 ---@param v any
 ---@return type type
+---@nodiscard
 function type(v) end
 
 ---#DES '_VERSION'
@@ -244,7 +269,7 @@ function warn(message, ...) end
 function xpcall(f, err) end
 ---#else
 ---#DES 'xpcall>5.2'
----@param f     function
+---@param f     async fun()
 ---@param msgh  function
 ---@param arg1? any
 ---@return boolean success
@@ -255,7 +280,10 @@ function xpcall(f, msgh, arg1, ...) end
 
 ---@version 5.1
 ---#DES 'unpack'
----@param list table
+---@generic T
+---@param list T[]
 ---@param i?   integer
 ---@param j?   integer
+---@return T
+---@nodiscard
 function unpack(list, i, j) end
