@@ -5,7 +5,6 @@ local vm             = require 'vm'
 local util           = require 'utility'
 local guide          = require 'parser.guide'
 local converter      = require 'proto.converter'
-local infer          = require 'vm.infer'
 local config         = require 'config'
 local linkedTable    = require 'linked-table'
 
@@ -16,8 +15,24 @@ local Care = util.switch()
         if not options.variable then
             return
         end
-        local isLib = vm.isGlobalLibraryName(source[1])
-        local isFunc = infer.getInfer(source):hasFunction()
+
+        local name = source[1]
+        local isLib = options.libGlobals[name]
+        if isLib == nil then
+            isLib = false
+            local global = vm.getGlobal('variable', name)
+            if global then
+                local uri = guide.getUri(source)
+                for _, set in ipairs(global:getSets(uri)) do
+                    if vm.isMetaFile(guide.getUri(set)) then
+                        isLib = true
+                        break
+                    end
+                end
+            end
+            options.libGlobals[name] = isLib
+        end
+        local isFunc = vm.getInfer(source):hasFunction()
 
         local type = isFunc and define.TokenTypes['function'] or define.TokenTypes.variable
         local modifier = isLib and define.TokenModifiers.defaultLibrary or define.TokenModifiers.static
@@ -66,7 +81,7 @@ local Care = util.switch()
                 return
             end
         end
-        if infer.getInfer(source):hasFunction() then
+        if vm.getInfer(source):hasFunction() then
             results[#results+1] = {
                 start      = source.start,
                 finish     = source.finish,
@@ -92,6 +107,7 @@ local Care = util.switch()
         }
     end)
     : case 'local'
+    : case 'self'
     : case 'getlocal'
     : case 'setlocal'
     : call(function (source, options, results)
@@ -164,23 +180,23 @@ local Care = util.switch()
         -- 5. Class declaration
             -- only search this local
         if loc.bindDocs then
-            for i, doc in ipairs(loc.bindDocs) do
-                if doc.type == "doc.class" and doc.bindSources then
-                    for _, src in ipairs(doc.bindSources) do
-                        if src == loc then
-                            results[#results+1] = {
-                                start      = source.start,
-                                finish     = source.finish,
-                                type       = define.TokenTypes.class,
-                            }
-                            return
-                        end
+            local isParam = source.parent.type == 'funcargs'
+                         or source.parent.type == 'in'
+            if not isParam then
+                for _, doc in ipairs(loc.bindDocs) do
+                    if doc.type == 'doc.class' then
+                        results[#results+1] = {
+                            start      = source.start,
+                            finish     = source.finish,
+                            type       = define.TokenTypes.class,
+                        }
+                        return
                     end
                 end
             end
         end
         -- 6. References to other functions
-        if infer.getInfer(loc):hasFunction() then
+        if vm.getInfer(loc):hasFunction() then
             results[#results+1] = {
                 start      = source.start,
                 finish     = source.finish,
@@ -651,6 +667,14 @@ local Care = util.switch()
             type   = define.TokenTypes.keyword,
         }
     end)
+    : case 'doc.cast.name'
+    : call(function (source, options, results)
+        results[#results+1] = {
+            start      = source.start,
+            finish     = source.finish,
+            type       = define.TokenTypes.variable,
+        }
+    end)
 
 local function buildTokens(uri, results)
     local tokens = {}
@@ -780,6 +804,7 @@ return function (uri, start, finish)
         uri        = uri,
         state      = state,
         text       = files.getText(uri),
+        libGlobals = {},
         variable   = config.get(uri, 'Lua.semantic.variable'),
         annotation = config.get(uri, 'Lua.semantic.annotation'),
         keyword    = config.get(uri, 'Lua.semantic.keyword'),
@@ -793,27 +818,26 @@ return function (uri, start, finish)
 
     for _, comm in ipairs(state.comms) do
         if start <= comm.start and comm.finish <= finish then
-            if comm.type == 'comment.short' then
-                local head = comm.text:match '^%-%s*[@|]'
-                if head then
-                    results[#results+1] = {
-                        start  = comm.start,
-                        finish = comm.start + #head + 1,
-                        type   = define.TokenTypes.comment,
-                    }
-                    results[#results+1] = {
-                        start      = comm.start + #head + 1,
-                        finish     = comm.start + #head + 2 + #comm.text:match('%S*', #head + 1),
-                        type       = define.TokenTypes.keyword,
-                        modifieres = define.TokenModifiers.documentation,
-                    }
+            local headPos = (comm.type == 'comment.short' and comm.text:match '^%-%s*[@|]()')
+                         or (comm.type == 'comment.long'  and comm.text:match '^@()')
+            if headPos then
+                local atPos
+                if comm.type == 'comment.short' then
+                    atPos = headPos + 2
                 else
-                    results[#results+1] = {
-                        start  = comm.start,
-                        finish = comm.finish,
-                        type   = define.TokenTypes.comment,
-                    }
+                    atPos = headPos + #comm.mark
                 end
+                results[#results+1] = {
+                    start  = comm.start,
+                    finish = comm.start + atPos - 2,
+                    type   = define.TokenTypes.comment,
+                }
+                results[#results+1] = {
+                    start      = comm.start + atPos - 2,
+                    finish     = comm.start + atPos - 1 + #comm.text:match('%S*', headPos),
+                    type       = define.TokenTypes.keyword,
+                    modifieres = define.TokenModifiers.documentation,
+                }
             else
                 results[#results+1] = {
                     start  = comm.start,
