@@ -19,6 +19,8 @@ local json       = require 'json'
 local scope      = require 'workspace.scope'
 local furi       = require 'file-uri'
 local inspect    = require 'inspect'
+local markdown   = require 'provider.markdown'
+local guide      = require 'parser.guide'
 
 ---@async
 local function updateConfig(uri)
@@ -258,6 +260,10 @@ m.register 'textDocument/didChange' {
     ---@async
     function (params)
         local doc     = params.textDocument
+        local scheme = furi.split(doc.uri)
+        if scheme ~= 'file' then
+            return
+        end
         local changes = params.contentChanges
         local uri     = files.getRealUri(doc.uri)
         workspace.awaitReady(uri)
@@ -519,7 +525,8 @@ m.register 'textDocument/completion' {
             local count, max = workspace.getLoadingProcess(uri)
             return {
                 {
-                    label = lang.script('HOVER_WS_LOADING', count, max),textEdit         = {
+                    label = lang.script('HOVER_WS_LOADING', count, max),
+                    textEdit    = {
                         range   = {
                             start   = params.position,
                             ['end'] = params.position,
@@ -984,23 +991,11 @@ m.register 'window/workDoneProgress/cancel' {
     end
 }
 
-m.register '$/didChangeVisibleRanges' {
-    ---@async
-    function (params)
-        local uri = files.getRealUri(params.uri)
-        await.close('visible:' .. uri)
-        await.setID('visible:' .. uri)
-        await.delay()
-        files.setVisibles(uri, params.ranges)
-    end
-}
-
 m.register '$/status/click' {
     ---@async
     function ()
-        -- TODO: translate
-        local titleDiagnostic = '进行工作区诊断'
-        local result = client.awaitRequestMessage('Info', 'xxx', {
+        local titleDiagnostic = lang.script.WINDOW_LUA_STATUS_DIAGNOSIS_TITLE
+        local result = client.awaitRequestMessage('Info', lang.script.WINDOW_LUA_STATUS_DIAGNOSIS_MSG, {
             titleDiagnostic,
         })
         if not result then
@@ -1156,55 +1151,60 @@ m.register '$/requestHint' {
     end
 }
 
--- Hint
-do
+m.register 'textDocument/inlayHint' {
+    capability = {
+        inlayHintProvider = {
+            resolveProvider = true,
+        },
+    },
     ---@async
-    local function updateHint(uri)
+    function (params)
+        local uri  = files.getRealUri(params.textDocument.uri)
         if not config.get(uri, 'Lua.hint.enable') then
             return
         end
-        local id = 'updateHint' .. uri
-        await.close(id)
-        await.setID(id)
         workspace.awaitReady(uri)
-        local visibles = files.getVisibles(uri)
-        if not visibles then
-            return
+        local core = require 'core.hint'
+        local start, finish = converter.unpackRange(uri, params.range)
+        local results = core(uri, start, finish)
+        local hintResults = {}
+        for i, res in ipairs(results) do
+            hintResults[i] = {
+                label        = {
+                    {
+                        value    = res.text,
+                        tooltip  = res.tooltip,
+                        location = res.source and converter.location(
+                                    guide.getUri(res.source),
+                                    converter.packRange(
+                                        guide.getUri(res.source),
+                                        res.source.start,
+                                        res.source.finish
+                                    )
+                                ),
+                    },
+                },
+                position     = converter.packPosition(uri, res.offset),
+                kind         = res.kind,
+                paddingLeft  = true,
+                paddingRight = true,
+            }
         end
-        await.close(id)
-        await.setID(id)
-        await.delay()
-        workspace.awaitReady(uri)
-        local edits = {}
-        local hint = require 'core.hint'
-        local _ <close> = progress.create(uri, lang.script.WINDOW_PROCESSING_HINT, 0.5)
-        for _, visible in ipairs(visibles) do
-            local piece = hint(uri, visible.start, visible.finish)
-            if piece then
-                for _, edit in ipairs(piece) do
-                    edits[#edits+1] = {
-                        text = edit.text,
-                        pos  = converter.packPosition(uri, edit.offset),
-                    }
-                end
-            end
-        end
-
-        proto.notify('$/hint', {
-            uri   = uri,
-            edits = edits,
-        })
+        return hintResults
     end
+}
 
-    files.watch(function (ev, uri)
-        if ev == 'update'
-        or ev == 'updateVisible' then
-            await.call(function () ---@async
-                updateHint(uri)
-            end)
-        end
-    end)
-end
+m.register 'inlayHint/resolve' {
+    capability = {
+        inlayHintProvider = {
+            resolveProvider = true,
+        },
+    },
+    ---@async
+    function (hint)
+        return hint
+    end
+}
 
 local function refreshStatusBar()
     local valid = config.get(nil, 'Lua.window.statusBar')
